@@ -7,7 +7,7 @@ use Types::Standard qw/Maybe/;
 use Types::Path::Tiny qw/Path/;
 use String::ShellQuote;
 
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 =head1 NAME
 
@@ -40,21 +40,39 @@ problem cpanm logs at a verbose level to a logfile.
 This module provides infrastructure so developers can easily add similar
 functionality to their command line applications.
 
-Shell::Carapace is mostly a very small wrapper around Capture::Tiny.
+Shell::Carapace is mostly a small wrapper around Capture::Tiny.
 
 =head1 ERROR HANDLING
 
 local() and remote() both die if a command fails by returning a positive exit
 code. 
 
+=head1 CAVEATS
+
+There isn't a good Perly way to tee output to both stdout and a log file.  To
+enable this feature this module pipes your cmd to "tee -a $logfile".  This will
+fail if you don't have tee in your $PATH.
+
+You can disable this behavior by setting the 'tee_logfile' attribute to false.
+In that case, command output will get written to the logfile only after the
+command completes instead of in real time.
+
+Doesn't work on win32.
+
 =cut
 
-has verbose     => (is => 'rw', default => sub { 0 });
-has print_cmd   => (is => 'ro', default => sub { 0 });
+has verbose     => (is => 'rw',   default => sub { 0 });
+has print_cmd   => (is => 'ro',   default => sub { 0 });
 has ssh_cmd     => (is => 'lazy', default => sub { '/usr/bin/ssh' });
 has ssh_options => (is => 'lazy', default => sub { [] });
-has logfile     => (is => 'rw', isa => Maybe[Path], coerce => Path->coercion, clearer => 1);
 has noop        => (is => 'rw', default => sub { 0 });
+has tee_logfile => (is => 'rw', default => sub { 1 });
+has logfile     => (
+    is      => 'rw',
+    isa     => Maybe[Path],
+    coerce  => Path->coercion,
+    clearer => 1,
+);
 
 before logfile => sub {
     my ($self, $logfile) = @_;
@@ -72,7 +90,7 @@ sub remote {
     my ($self, $user, $host, @cmd) = @_;
 
     my $ssh_opts = $self->ssh_options;
-    my @ssh_cmd  = (scalar @cmd == 1)
+    my @ssh_cmd  = @cmd == 1
         ? (join(" ", $self->ssh_cmd, "$user\@$host", @$ssh_opts, @cmd))
         : ($self->ssh_cmd, "$user\@$host", @$ssh_opts, @cmd);
 
@@ -82,20 +100,34 @@ sub remote {
 sub local {
     my ($self, @cmd) = @_; 
 
-    say ">> " . join(" ", @cmd) if $self->verbose || $self->print_cmd;
+    say ">> " . $self->_stringify(@cmd) if $self->verbose || $self->print_cmd;
 
-    if ($self->noop) {
-        return $cmd[0] if scalar @cmd == 1;
-        return join(" ", shell_quote(@cmd));
+    return $self->_stringify(@cmd) if $self->noop;
+
+    my $merged_out;
+    my $exit;
+
+    if ($self->logfile && $self->tee_logfile) {
+        my $cmd_str = $self->_stringify(@cmd);
+        $self->logfile->touchpath;
+        $self->logfile->append_utf8(">> $cmd_str\n");
+
+        $cmd_str = "($cmd_str) | tee -a " . shell_quote($self->logfile);
+
+        ($merged_out, $exit) = $self->verbose
+            ? tee_merged     { system $cmd_str }
+            : capture_merged { system $cmd_str };
+    }
+    else {
+        ($merged_out, $exit) = $self->verbose
+            ? tee_merged     { system @cmd }
+            : capture_merged { system @cmd };
     }
 
-    my ($merged_out, $exit) = $self->verbose
-        ? tee_merged     { system @cmd }
-        : capture_merged { system @cmd };
-
-    if ($self->logfile) {
+    if ($self->logfile && !$self->tee_logfile) {
+        my $cmd_str = $self->_stringify(@cmd);
         $self->logfile->touchpath;
-        $self->logfile->append_utf8(">> " . join(" ", @cmd) . "\n");
+        $self->logfile->append_utf8(">> $cmd_str\n");
         $self->logfile->append_utf8($merged_out);
     }
 
@@ -104,6 +136,12 @@ sub local {
     return $merged_out;
 }
 
+sub _stringify {
+    my ($self, @cmd) = @_;
+    return $cmd[0] if @cmd == 1;
+    return join(" ", shell_quote @cmd);
+}
+
 1;
 
 =head1 SEE ALSO
